<?php
/*
 * Benno MailArchiv
 *
 * Copyright  2008-2012 LWsystems GmbH & Co. KG
 *
 * http://www.lw-systems.de/
 * http://www.benno-mailarchiv.de/
 *
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License version 3
 * as published by the Free Software Foundation.
 *
 * Binary versions of this file provided by LWsystems to you as
 * well as other copyrighted, protected or trademarked materials like
 * logos, graphics, fonts, specific documentations and configurations,
 * cryptographic keys etc. are subject to a license agreement between
 * you and LWsystems.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You can find a copy of the GNU Affero General Public License at
 * this URI:
 * http://www.gnu.org/licenses/agpl-3.0.html
 * If not, write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 330, Boston, MA  02111-1307  USA
 */


class EMail
{

    /** id
     *
     * session mail id: uniqe id of this email in this session
     *
     * @access public
     * @var string
     */
    var $id;


    /** archive
     *
     * archive of this email
     *
     * @access public
     * @var string
     */
    var $archive;

    /** bennoId
     *
     * uniqe id in archive: <box>:<checksum>
     *
     * @access public
     * @var string
     */
    var $bennoId;

    var $from;
    var $to;
    var $cc;
    var $bcc;

    // array
    var $_lto;
    var $_lcc;
    var $_lbcc;

    var $subject;
    var $date;
    var $size;
    var $body;

    var $hasAttachment;
    var $attachments;
    var $verified;
    var $dateTS;


    function EMail ($id,$archive,$bennoId='',$from='',$to=array(),$cc=array(),$bcc=array(),$subject='',$date='',$size='',$body='',$hasAttachment=false,$verified=false,$dateTS=0)
    {
        $this->id       = $id; 
        $this->archive  = $archive; 
        $this->bennoId  = $bennoId; 
        $this->from     = $from;
        $this->_lto     = $to;
        $this->_lcc     = $cc;
        $this->_lbcc    = $bcc;
        $this->to       = $this->getTo();
        $this->cc       = $this->getCc();
        $this->bcc      = $this->getBcc();
        $this->subject  = $subject;
        if ( !is_numeric($date) )
            $this->date = $date;
        else
            $this->date = $date." UTC";
        $this->size     = $size;
        $this->body     = $body;
        $this->hasAttachment = $hasAttachment;

        $this->attachments = array();
        $this->verified = $verified;
        $this->dateTS = $dateTS;
    }

    /**
     * Returns date as epoch integer
     *
     * @access public
     * @return int $epoch
     */
    function getDate ()
    {
        $retDate = strtotime($this->date);
        if (!$retDate) {
            $retDate = $this->dateTS;
        }
        return $retDate;
    }


    /**
     * Returns string with header field
     *
     * @access public
     * @return string $body
     */
    function getFrom ()
    {
        return $this->from;
    }


    /**
     * Returns string with header field
     *
     * @access public
     * @return string $body
     */
    function getSubject()
    {
        return $this->subject;
    }


    /**
     * Returns string with header field
     *
     * @access public
     * @return string $body
     */
    function getTo ()
    {
        $return = '';
        foreach ($this->_lto as $value) {
            $return .= $value.' ';
        }
        return $return;
    }


    /**
     * Returns string with header field
     *
     * @access public
     * @return string $body
     */
    function getCc ()
    {
        $return = '';
        foreach ($this->_lcc as $value) {
            $return .= $value.' ';
        }
        return $return;
    }


    /**
     * Returns string with header field
     *
     * @access public
     * @return string $body
     */
    function getBcc ()
    {
        $return = '';
        foreach ($this->_lbcc as $value) {
            $return .= $value.' ';
        }
        return $return;
    }


    /**
     * Returns mail body in plain text
     *
     * @access public
     * @param string $contentType
     * @return string $body
     */
    function getBody($contentType='text/plain')
    {
        global $App;

        /*
            $this->body = Array(
                Array( 'content-type' => 'text/plain', 'data'=> 'Text ...'),
                Array(...),
                Array(...)
            );
        */
        $body['text/plain'] = '';
        $body['text/html'] = '';
        $body['unknown'] = '';
        $count['text/plain'] = 0;
        $count['text/html'] = 0;
        if (isset($this->body)) {
            // handle disclaimer
            foreach ($this->body as $bodypart) {
                $partContentType = $bodypart['content-type'];
                // Content-Type == text/html
                if (stripos($partContentType,'text/html') !== false) {
                    $body['text/html'] .= $bodypart['data']."\n";
                    if ($count['text/html'] == 1) {
                        $Attachment = new Attachment('content.disclaimer',$partContentType,0,false);
                        $this->addAttachment($Attachment);
                    }
                    continue;
                }
                // Content-Type == text/plain
                if (stripos($partContentType,'text/plain') !== false) {

                    // unconverted html could break user interface
                    // ... <base href="$external_site" >
                    $body['text/plain'] .= htmlspecialchars($bodypart['data'],ENT_COMPAT | ENT_HTML401,'UTF-8')."\n";

                    // second body part text/plain contains normally disclaimer
                    if ($count['text/plain'] == 1) {    // disclaimer part
                        $Attachment = new Attachment('content.disclaimer',$partContentType,0,false);
                        $this->addAttachment($Attachment);
                    }
                    $count['text/plain']++;
                    continue;
                }
                $body['unknown'] = $bodypart['data'];
            }

            $content = $body[$contentType];
            if (!$content) {
                $App->log('[model/EMail] Only text/plain body found.',1,'EMail');
                throw new Exception('NO_CONTENT');
            }
        }                               // text/plain
        return $content;
    }


    /**
     * Return array with attachments
     */
    function getAttachments()
    {
        return $this->attachments;
    }


    /**
     * Return checksum test result
     */
    function isVerified()
    {
        return $this->verified;
    }


    /**
     * Add attachment object to attachment list
     */
    function addAttachment($attachment)
    {
        $this->hasAttachment = true;
        array_push($this->attachments,$attachment);
    }

}
?>
